@@ -14,6 +14,7 @@ def _(msg):
     return gettext.gettext(msg)
 
 # Aliasing commonly used classes
-from specter.spec import Describe, DataDescribe, Spec, fixture  # NOQA
-from specter.expect import (expect, require, skip, skip_if,  # NOQA
-                            incomplete, metadata)  # NOQA
+from specter.spec import Describe, Spec, fixture   # NOQA
+from specter.spec import DataDescribe, DataSpec  # NOQA
+from specter.expect import expect, require, skip, skip_if  # NOQA
+from specter.expect import incomplete, metadata  # NOQA
