@@ -5,6 +5,7 @@ from coverage import coverage
 from specter import _
 from specter.scanner import SuiteScanner
 from specter.reporting import ReporterPluginManager
+from specter.parallel import ParallelManager
 
 
 class SpecterRunner(object):
@@ -18,25 +19,26 @@ class SpecterRunner(object):
         self.arg_parser = ArgumentParser(description=self.DESCRIPTION)
         self.setup_argparse()
         self.suites = []
-        self.reporter_manager = ReporterPluginManager()
+        self.reporter_manager = None
+        self.parallel_manager = None
 
     def setup_argparse(self):
         self.arg_parser.add_argument(
             '--coverage', dest='coverage', action='store_true',
             help=_('Activates coverage.py integration'))
         self.arg_parser.add_argument(
-            '--search', type=str, dest='search',
+            '--search', type=str, dest='search', metavar='',
             help=_('The spec suite folder path.'))
         self.arg_parser.add_argument(
             '--no-art', dest='no_art', action='store_true',
             help=_('Disables ASCII art'))
         self.arg_parser.add_argument(
-            '--select-module', dest='select_module',
+            '--select-module', dest='select_module', metavar='',
             help=_('Selects a module path to run. '
                    'Ex: spec.sample.TestClass'),
             default=None)
         self.arg_parser.add_argument(
-            '--select-by-metadata', dest='select_meta',
+            '--select-by-metadata', dest='select_meta', metavar='',
             help=_('Selects tests to run by specifying a list of '
                    'key=value pairs you wish to run'),
             default=[], nargs='*')
@@ -46,6 +48,13 @@ class SpecterRunner(object):
         self.arg_parser.add_argument(
             '--xunit-results', dest='xunit_results', metavar='',
             help=_('Saves out xUnit compatible results to a specifed file'))
+        self.arg_parser.add_argument(
+            '--parallel', dest='parallel', action='store_true',
+            help=_('Activate parallel testing mode'))
+        self.arg_parser.add_argument(
+            '--num-processes', dest='num_processes', default=6, metavar='',
+            help=_('Specifies the number of processes to use under '
+                   'parallel mode (default: 6)'))
 
     def generate_ascii_art(self):
         tag_line = _('Keeping the boogy man away from your code!')
@@ -60,13 +69,36 @@ class SpecterRunner(object):
     """.format(tag=tag_line)
         return ascii_art
 
+    def get_coverage_omit_list(self):
+        omit_list = ['*/pyevents/event.py',
+                     '*/pyevents/manager.py',
+                     '*/specter/spec.py',
+                     '*/specter/expect.py',
+                     '*/specter/parallel.py',
+                     '*/specter/runner.py',
+                     '*/specter/util.py',
+                     '*/specter/reporting/__init__.py',
+                     '*/specter/reporting/console.py',
+                     '*/specter/reporting/dots.py',
+                     '*/specter/reporting/xunit.py',
+                     '*/specter/__init__.py']
+        return omit_list
+
     def run(self, args):
         select_meta = None
         self.arguments = self.arg_parser.parse_args(args)
 
+        self.reporter_manager = ReporterPluginManager()
+
         # Let each reporter parse cli arguments
         self.reporter_manager.process_arguments(self.arguments)
 
+        if self.arguments.parallel:
+            self.parallel_manager = ParallelManager(
+                num_processes=self.arguments.num_processes,
+                track_coverage=self.arguments.coverage,
+                coverage_omit=self.get_coverage_omit_list())
+
         if self.arguments.select_meta:
             metas = [meta.split('=') for meta in self.arguments.select_meta]
             select_meta = {meta[0]: meta[1].strip('"\'') for meta in metas}
@@ -76,36 +108,34 @@ class SpecterRunner(object):
 
         if self.arguments.coverage:
             print(_(' - Running with coverage enabled - '))
-            self.coverage = coverage(omit=['*/pyevents/event.py',
-                                           '*/pyevents/manager.py',
-                                           '*/specter/spec.py',
-                                           '*/specter/expect.py',
-                                           '*/specter/reporting/__init__.py',
-                                           '*/specter/reporting/console.py',
-                                           '*/specter/__init__.py'])
+            self.coverage = coverage(omit=self.get_coverage_omit_list(),
+                                     data_suffix=self.arguments.parallel)
             self.coverage._warn_no_data = False
+            self.coverage.start()
 
         self.suite_types = self.suite_scanner.scan(
             search_path=self.arguments.search,
             module_name=self.arguments.select_module)
 
+        # Serial: Add and Execute | Parallel: Collect all with the add process
         for suite_type in self.suite_types:
-            # Start Coverage Capture
-            if self.coverage:
-                self.coverage.start()
 
             suite = suite_type()
             self.suites.append(suite)
             self.reporter_manager.subscribe_all_to_describe(suite)
-            suite.execute(select_metadata=select_meta)
+            suite.execute(select_metadata=select_meta,
+                          parallel_manager=self.parallel_manager)
 
-            # Start Coverage Capture
-            if self.coverage:
-                self.coverage.stop()
+        # Actually execute the tests for parallel now
+        if self.arguments.parallel:
+            self.parallel_manager.execute_all()
 
         # Save coverage data if enabled
         if self.coverage:
+            self.coverage.stop()
             self.coverage.save()
+            if self.arguments.parallel:
+                self.coverage.combine()
 
         # Print all console summaries
         for reporter in self.reporter_manager.get_console_reporters():
@@ -115,7 +145,7 @@ class SpecterRunner(object):
         self.suite_scanner.destroy()
 
 
-def activate(): #pragma: no cover
+def activate():  # pragma: no cover
     args = sys.argv[1:]
     runner = SpecterRunner()
     runner.run(args)
