@@ -2,6 +2,7 @@ import inspect
 import re
 import itertools
 import sys
+import six
 
 
 def convert_camelcase(input_str):
@@ -12,11 +13,19 @@ def convert_camelcase(input_str):
     return re.sub(camelcase_tags, r' \1', input_str)
 
 
-def get_called_src_line():
-    src_line = None
+def get_called_src_line(steps=2, use_child_attr=None):
+    src_line, last_frame = None, inspect.currentframe()
+
+    for i in range(steps):
+        last_frame = last_frame.f_back
+
+    self = module = last_frame.f_locals['self']
+    # Use an attr instead of self
+    if use_child_attr:
+        module = getattr(self, use_child_attr)
+
     try:
-        last_frame = inspect.currentframe().f_back.f_back
-        last_module = inspect.getmodule(type(last_frame.f_locals['self']))
+        last_module = inspect.getmodule(type(module))
         line = last_frame.f_lineno - 1
         src_line = inspect.getsourcelines(last_module)[0][line]
     except:
@@ -32,10 +41,15 @@ def get_expect_param_strs(src_line):
 def get_source_from_frame(frame):
     self = frame.f_locals.get('self', None)
     cls = frame.f_locals.get('cls', None)
-    module = inspect.getmodule(type(self) if self else cls)
-    module_path = module.__file__
-    lines = inspect.getsourcelines(module)[0]
-    return lines, module_path
+    insp_obj = inspect.getmodule(type(self) if self else cls) or frame.f_code
+
+    line_num_modifier = 0
+    if inspect.iscode(insp_obj):
+        line_num_modifier -= 1
+
+    module_path = inspect.getfile(insp_obj)
+    source_lines = inspect.getsourcelines(insp_obj)
+    return source_lines[0], module_path, source_lines[1] + line_num_modifier
 
 
 def get_all_tracebacks(tb, tb_list=[]):
@@ -46,14 +60,14 @@ def get_all_tracebacks(tb, tb_list=[]):
     return tb_list
 
 
-def get_numbered_source(lines, line_num):
+def get_numbered_source(lines, line_num, starting_line=0):
     try:
-        center = line_num - 1
+        center = (line_num - starting_line) - 1
         start = center - 2 if center - 2 > 0 else 0
         end = center + 2 if center + 2 <= len(lines) else len(lines)
 
         orig_src_lines = [line.rstrip('\n') for line in lines[start:end]]
-        line_range = range(start+1, end+1)
+        line_range = range(start+1+starting_line, end+1+starting_line)
         nums_and_source = zip(line_range, orig_src_lines)
 
         traceback_lines = []
@@ -75,10 +89,11 @@ def get_real_last_traceback(exception):
     tb_list = get_all_tracebacks(exc_traceback)[1:]
 
     for traceback in tb_list:
-        lines, last_module_path = get_source_from_frame(traceback.tb_frame)
-        traceback_lines = get_numbered_source(lines, traceback.tb_lineno)
+        lines, path, line_num = get_source_from_frame(traceback.tb_frame)
+        traceback_lines = get_numbered_source(lines, traceback.tb_lineno,
+                                              line_num)
 
-        traceback_lines.insert(0, '  - {0}'.format(last_module_path))
+        traceback_lines.insert(0, '  - {0}'.format(path))
         traceback_lines.insert(1, '  ------------------')
         traceback_lines.append('  ------------------')
         traceback_blocks.append(traceback_lines)
@@ -91,13 +106,13 @@ def get_real_last_traceback(exception):
 
 
 def find_by_metadata(meta, cases):
-    selected_cases = []
-    for case in cases:
+    selected_cases = {}
+    for case_id, case in six.iteritems(cases):
         matched_keys = set(meta.keys()) & set(case.metadata.keys())
 
         for key in matched_keys:
             if meta.get(key) == case.metadata.get(key):
-                selected_cases.append(case)
+                selected_cases[case_id] = case
 
     return selected_cases
 
@@ -115,8 +130,17 @@ def children_with_tests_with_metadata(meta, describe):
 def extract_metadata(case_func):
     # Handle metadata decorator
     metadata = {}
-    if 'onCall' in case_func.__name__:
-        decorator_data = case_func()
-        case_func = decorator_data[0]
-        metadata = decorator_data[1]
+    if 'DECORATOR_ONCALL' in case_func.__name__:
+        try:
+            decorator_data = case_func()
+            case_func = decorator_data[0]
+            metadata = decorator_data[1]
+        except Exception as e:
+            # Doing this old school to avoid dependancy conflicts
+            handled = ['TestIncompleteException', 'TestSkippedException']
+            if type(e).__name__ in handled:
+                case_func = e.func
+                metadata = e.other_data.get('metadata')
+            else:
+                raise e
     return case_func, metadata
